"use strict";
/*
* This program and the accompanying materials are made available under the terms of the
* Eclipse Public License v2.0 which accompanies this distribution, and is available at
* https://www.eclipse.org/legal/epl-v20.html
*
* SPDX-License-Identifier: EPL-2.0
*
* Copyright Contributors to the Zowe Project.
*
*/
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const utilities_1 = require("../../../../../utilities");
const config_1 = require("../../../../../config");
const ConfigBuilder_1 = require("../../../../../config/src/ConfigBuilder");
const security_1 = require("../../../../../security");
const ConfigUtils_1 = require("../../../../../config/src/ConfigUtils");
const OverridesLoader_1 = require("../../../OverridesLoader");
/**
 * Init config
 */
class InitHandler {
    /**
     * Process the command and input.
     *
     * @param {IHandlerParameters} params Parameters supplied by yargs
     *
     * @throws {ImperativeError}
     */
    process(params) {
        return __awaiter(this, void 0, void 0, function* () {
            this.params = params;
            // Load the config and set the active layer according to user options
            yield this.ensureCredentialManagerLoaded();
            const config = utilities_1.ImperativeConfig.instance.config;
            const configDir = params.arguments.globalConfig ? null : process.cwd();
            config.api.layers.activate(params.arguments.userConfig, params.arguments.globalConfig, configDir);
            const layer = config.api.layers.get();
            yield this.initWithSchema(config, params.arguments.userConfig);
            if (params.arguments.prompt !== false && !security_1.CredentialManagerFactory.initialized && config.api.secure.secureFields().length > 0) {
                const warning = ConfigUtils_1.secureSaveError();
                params.response.console.log(utilities_1.TextUtils.chalk.yellow("Warning:\n") +
                    `${warning.message} Skipped prompting for credentials.\n\n${warning.additionalDetails}\n`);
            }
            // Write the active created/updated config layer
            yield config.save(false);
            params.response.console.log(`Saved config template to ${layer.path}`);
        });
    }
    /**
     * If CredentialManager was not already loaded by Imperative.init, load it
     * now before performing config operations in the init handler.
     */
    ensureCredentialManagerLoaded() {
        return __awaiter(this, void 0, void 0, function* () {
            if (!security_1.CredentialManagerFactory.initialized) {
                yield OverridesLoader_1.OverridesLoader.loadCredentialManager(utilities_1.ImperativeConfig.instance.loadedConfig, utilities_1.ImperativeConfig.instance.callerPackageJson);
            }
        });
    }
    /**
     * Creates JSON template for config. Also creates a schema file in the same
     * folder alongside the config.
     * @param config Config object to be populated
     * @param user If true, properties will be left empty for user config
     */
    initWithSchema(config, user) {
        return __awaiter(this, void 0, void 0, function* () {
            // Build the schema and write it to disk
            config_1.ConfigSchema.updateSchema();
            const opts = {};
            if (!user) {
                opts.populateProperties = true;
                opts.getSecureValue = this.promptForProp.bind(this);
            }
            // Build new config and merge with existing layer
            const newConfig = yield ConfigBuilder_1.ConfigBuilder.build(utilities_1.ImperativeConfig.instance.loadedConfig, opts);
            config.api.layers.merge(newConfig);
        });
    }
    /**
     * Prompts for the value of a property on the CLI. Returns null if `--prompt false`
     * argument is passed, or prompt times out, or a blank value is entered.
     * @param propName The name of the property
     * @param property The profile property definition
     */
    promptForProp(propName, property) {
        var _a;
        return __awaiter(this, void 0, void 0, function* () {
            // skip prompting in CI environment
            if (this.params.arguments.prompt === false || !security_1.CredentialManagerFactory.initialized) {
                return null;
            }
            // get the summary and value
            if (((_a = property.optionDefinition) === null || _a === void 0 ? void 0 : _a.description) != null) {
                propName = `${propName} (${property.optionDefinition.description})`;
            }
            const propValue = yield this.params.response.console.prompt(`${propName} - blank to skip: `, { hideText: property.secure });
            // coerce to correct type
            if (propValue && propValue.trim().length > 0) {
                if (propValue === "true")
                    return true;
                if (propValue === "false")
                    return false;
                if (!isNaN(propValue) && !isNaN(parseFloat(propValue)))
                    return parseInt(propValue, 10);
            }
            return propValue || null;
        });
    }
}
exports.default = InitHandler;
//# sourceMappingURL=init.handler.js.map